#!/bin/bash 
PATH=$PATH:~/.platformio/packages/toolchain-xtensa/bin
COMPILERS=()
COMPILERS+=(`which g++`)
COMPILERS+=(`which clang++`)
COMPILERS+=(`which arm-linux-gnueabi-g++-8`)
COMPILERS+=(`which arm-linux-gnueabi-g++-6`)
COMPILERS+=(`which xtensa-lx106-elf-g++`)
COMPILERS+=(`which xtensa-esp32-elf-g++`)
COMPILERS+=(`which avr-g++`)
COMPILERS+=(`which sparc64-linux-elf-g++`)
COMPILERS+=(`which msp430-g++`)


echo found ${COMPILERS[*]}
BASEFLAGS="-DRUNTEST -c -fexceptions"
#OPTLEVELS=(-g -O -O2 -O3 -Os)
OPTLEVELS=(-O2 -O3 -Os)

if [ "$1" == "" ]; then
	echo syntax: $0 "<ccfile.cc>"
	exit
fi

FILE=$1

if [ ! -d obj ]; then
	mkdir obj
fi

CSV=`echo $FILE | sed 's/.cc//;s/.c//'`.csv

echo "Compiler;Flags;Function;Size" > $CSV

for COMPILER in "${COMPILERS[@]}"; do
	MACH=`$COMPILER -dumpmachine`
	echo $MACH
	for OPTLEVEL in "${OPTLEVELS[@]}"; do
		OBJFILE=`echo obj/$FILE | sed 's/.cc//;s/.c//'`
		OBJFILE+="-$MACH$OPTLEVEL.o"
		$COMPILER $BASEFLAGS $OPTLEVEL $FILE -o $OBJFILE
		export COMPILER OPTLEVEL
		nm -C -S -t d $OBJFILE |
		grep -v __PRETTY_FUNCTION__ |
		grep -e'_[A-Za-z0-9]\+(' -e'_[A-Za-z0-9]\+$' |
		grep --regexp ' T ' |
		sed 's/(.*//;s/.*\.o:[0-9a-f]* //;s/ T /\t/;s/ W /\t/;s/\(.*\)_\([0-9a-zA-Z]\+\)/\1;\2/' |
		awk 'BEGIN{FS=" "} {printf("%s;%d\n",$3,$2);}' |
		sed "s!^!${MACH};@${OPTLEVEL};!" >> $CSV
	done
done

echo done writing $CSV
