/*
 *  Copyright (C) 2017-2018, Thomas Maier-Komor
 *
 *  This source file belongs to Wire-Format-Compiler.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "mangler.h"
#include <assert.h>
#if defined __x86 || defined __x86_64
#include <x86intrin.h>
#endif

#ifndef RUNTEST
#define WFC
#endif

#ifdef __AVR__
typedef int16_t ssize_t;
#define VARINTBITS 16
#elif defined __XTENSA__
typedef int32_t ssize_t;
#endif


#ifdef RUNTEST
#include <unistd.h> // for _POSIX_TIMERS
#include <string.h>
#ifdef __AVR__
#define wire2varint wire2varint16_0
#define wire2varint_0 wire2varint16_0
#define varint2mem varint2mem16_0 
#define varint2mem_0 varint2mem16_0 
#elif defined __XTENSA__
#define wire2varint wire2varint32_0
#define wire2varint_0 wire2varint32_0
#define wire2varint_1 wire2varint32_1
#define wire2varint_2 wire2varint32_2
#define wire2varint_3 wire2varint32_3
#define varint2mem varint2mem32_0 
#define varint2mem_0 varint2mem32_0
#define varint2mem_1 varint2mem32_1
#else
#define wire2varint wire2varint64_0
#define varint2mem varint2mem64_0 
#endif
#else
//#define V2W1	-- generated 
//#define W2V0
//#define WSU8_0
//#define WSU16_0
//#define WSU32_0
//#define WSU64_0
//#define W2V32_0
//#define W2V16_0
#endif

#ifdef WFC
#define W2V0
#define WSU64_0
#endif

#if VARINTBITS==64
#define wire2varint64 wire2varint
#define varint2mem64 varint2mem
#define varint2wire64 varint2wire
//typedef uint64_t varint_t;
#elif VARINTBITS==32
#define wire2varint32 wire2varint
#define varint2mem32 varint2mem
#define varint2wire32 varint2wire
//typedef uint32_t varint_t;
#elif VARINTBITS==16
#define wire2varint16 wire2varint
#define varint2mem16 varint2mem
#define varint2wire16 varint2wire
//typedef uint16_t varint_t;
#else
#error invalid varint_t bit-size
#endif


/* wire to varint with varint_t is uint64_t */

//#ifndef RUNTEST
//int wire2varint64(const uint8_t *wire, ssize_t wl, uint64_t *r)
//#endif
#ifdef WFC
int wire2varint(const uint8_t *wire, signed wl, varint_t *r)
#endif

#ifdef RUNTEST
int wire2varint64_0(const uint8_t *wire, ssize_t wl, varint_t *r)
#endif
#if defined RUNTEST || defined W2V0
{
	uint8_t u8;
	int n = 0;
	*r = 0;
	do {
		if (--wl < 0)
			return 0;
		u8 = *wire++;
		*r |= (uint64_t)(u8&~0x80) << (n*7);
		++n;
	} while ((u8 & 0x80) && (n < 10));
	return n;
}
#endif


#ifdef RUNTEST
int wire2varint64_1(const uint8_t *wire, ssize_t wl, uint64_t *r)
#endif
#if defined RUNTEST || defined W2V1
{
	uint8_t u8;
	int n = 0;
	*r = 0;
	do {
		if (--wl < 0)
			return 0;
		u8 = *wire++;
		*r |= (uint64_t)(u8&~0x80) << (n*7);
		++n;
	} while ((u8 >> 7) && (n < 10));
	return n;
}
#endif


#ifdef RUNTEST
int wire2varint64_2(const uint8_t *wire, ssize_t wl, uint64_t *r)
#endif
#if defined RUNTEST || defined W2V2
{
	uint8_t u8;
	uint64_t res = 0;
	int n = 0;
	do {
		if (--wl < 0)
			return 0;
		u8 = *wire++;
		res |= (uint64_t)(u8&~0x80) << n;
		n += 7;
	} while ((u8 & 0x80) && (n < 70));
	*r = res;
	return n/7;
}
#endif


#ifdef RUNTEST
int wire2varint64_3(const uint8_t *wire, ssize_t wl, uint64_t *r)
#endif
#if defined RUNTEST || defined W2V3
{
	const uint8_t *w = wire;
	const uint8_t *e = wire;
	if (wl >= 10)	
		e += 10;
	else if (wl <= 0)
		return 0;
	else
		e += wl;
	uint8_t u8 = *w;
	*r = u8 & 0x7f;
	++w;
	int s = 7;
	while ((u8 & 0x80) && (w < e)) {
		u8 = *w++;
		*r |= (uint64_t)(u8 & 0x7f) << s;
		s += 7;
	}
	return w-wire;
}
#endif


/* TODO: optimized calling convention */
varint_t parse_varint_0(const uint8_t **wire, signed wl)
{
	const uint8_t *w = *wire;
	varint_t r = 0;
	uint8_t u8 = 0x80;
	int n = 0;
	while ((wl > 0) && (u8 & 0x80)) {
		u8 = *w++;
		r |= (uint64_t)(u8&~0x80) << n;
		n += 7;
		assert(n < 70);
	}
	*wire = w;
	return r;
}

/* TODO */
int parse_svarint_0(const uint8_t *wire, signed wl, varint_t *r)
{
	uint8_t u8;
	int n = 0;
	*r = 0;
	do {
		if (--wl < 0)
			return 0;
		u8 = *wire++;
		*r |= (uint64_t)(u8&0x7f) << (n*7);
		++n;
	} while ((u8 & 0x80) && (n < 10));
	return n;
}


/* wire to varint with varint_t is uint32_t */
//#ifndef RUNTEST
//int wire2varint32(const uint8_t *wire, ssize_t wl, uint32_t *r)
//#endif

#ifdef RUNTEST
int wire2varint32_0(const uint8_t *wire, ssize_t wl, uint32_t *r)
#endif
#if defined RUNTEST || defined W2V32_0
{
	uint8_t u8;
	int n = 0;
	*r = 0;
	do {
		if (wl-- <= 0)
			return 0;
		u8 = *wire++;
		*r |= (uint32_t)(u8&~0x80) << (n*7);
		++n;
	} while ((u8 & 0x80) && (n < 5));
	return n;
}
#endif


#ifdef RUNTEST
int wire2varint32_1(const uint8_t *wire, ssize_t wl, uint32_t *r)
#endif
#if defined RUNTEST || defined W2V32_1
{
	uint8_t u8;
	int n = 0;
	*r = 0;
	do {
		if (wl-- <= 0)
			return 0;
		u8 = *wire++;
		*r |= (uint32_t)(u8&~0x80) << (n*7);
		++n;
	} while ((u8 >> 7) && (n < 10));
	return n;
}
#endif


#ifdef RUNTEST
int wire2varint32_2(const uint8_t *wire, ssize_t wl, uint32_t *r)
#endif
#if defined RUNTEST || defined W2V32_2
{
	uint8_t u8;
	int n = 0;
	*r = 0;
	do {
		if (wl-- <= 0)
			return 0;
		u8 = *wire++;
		*r |= (uint64_t)(u8&~0x80) << n;
		n += 7;
	} while ((u8 & 0x80) && (n < 35));
	return n/7;
}
#endif


#ifdef RUNTEST
int wire2varint32_3(const uint8_t *wire, ssize_t wl, uint32_t *r)
#endif
#if defined RUNTEST || defined W2V32_3
{
	const uint8_t *w = wire;
	const uint8_t *e = wire;
	if (wl >= 10)	
		e += 10;
	else if (wl <= 0)
		return 0;
	else
		e += wl;
	uint8_t u8 = *w;
	*r = u8 & 0x7f;
	++w;
	int s = 7;
	while ((u8 & 0x80) && (w < e)) {
		u8 = *w++;
		*r |= (uint32_t)(u8 & 0x7f) << s;
		s += 7;
	}
	return w-wire;
}
#endif



/* wire to varint with varint_t is uint16_t */
//#ifndef RUNTEST
//int wire2varint16(const uint8_t *wire, ssize_t wl, uint16_t *r)
//#endif

#ifdef RUNTEST
int wire2varint16_0(const uint8_t *wire, ssize_t wl, uint16_t *r)
#endif
#if defined RUNTEST || defined W2V16_0
{
	uint8_t u8;
	int n = 0;
	*r = 0;
	do {
		if (wl-- <= 0)
			return 0;
		u8 = *wire++;
		*r |= (uint16_t)(u8&~0x80) << (n*7);
		++n;
	} while ((u8 & 0x80) && (n < 3));
	return n;
}
#endif


#ifdef RUNTEST
int wire2varint16_1(const uint8_t *wire, ssize_t wl, uint16_t *r)
#endif
#if defined RUNTEST || defined W2V16_1
{
	uint8_t u8;
	int n = 0;
	*r = 0;
	do {
		if (wl-- <= 0)
			return 0;
		u8 = *wire++;
		*r |= (uint16_t)(u8&~0x80) << (n*7);
		++n;
	} while ((u8 >> 7) && (n < 10));
	return n;
}
#endif


#ifdef RUNTEST
int wire2varint16_2(const uint8_t *wire, ssize_t wl, uint16_t *r)
#endif
#if defined RUNTEST || defined W2V16_2
{
	uint8_t u8;
	int n = 0;
	*r = 0;
	do {
		if (wl-- <= 0)
			return 0;
		u8 = *wire++;
		*r |= (uint64_t)(u8&~0x80) << n;
		n += 7;
	} while ((u8 & 0x80) && (n < 21));
	return n/7;
}
#endif


#ifdef RUNTEST
int wire2varint16_3(const uint8_t *wire, ssize_t wl, uint16_t *r)
#endif
#if defined RUNTEST || defined W2V16_3
{
	const uint8_t *w = wire;
	const uint8_t *e = wire;
	if (wl >= 3)	
		e += 3;
	else if (wl <= 0)
		return 0;
	else
		e += wl;
	uint8_t u8 = *w;
	*r = u8 & 0x7f;
	++w;
	int s = 7;
	while ((u8 & 0x80) && (w < e)) {
		u8 = *w++;
		*r |= (uint32_t)(u8 & 0x7f) << s;
		s += 7;
	}
	return w-wire;
}
#endif



////////////////
//
//
//

#ifdef RUNTEST
int varint2wire64(void (*put)(uint8_t), uint64_t v)
{
	int n = 0;
	do {
		uint8_t u8 = v & 0x7f;
		v >>= 7;
		if (v)
			u8 |= 0x80;
		put(u8);
		++n;
	} while (v);
	return n;
}


int varint2wire32(void (*put)(uint8_t), uint32_t v)
{
	int n = 0;
	do {
		uint8_t u8 = v & 0x7f;
		v >>= 7;
		if (v)
			u8 |= 0x80;
		put(u8);
		++n;
	} while (v);
	return n;
}


int varint2wire16(void (*put)(uint8_t), uint16_t v)
{
	int n = 0;
	do {
		uint8_t u8 = v & 0x7f;
		v >>= 7;
		if (v)
			u8 |= 0x80;
		put(u8);
		++n;
	} while (v);
	return n;
}
#endif




/////////////


//#ifndef RUNTEST
//int varint2mem64(uint8_t *wire, ssize_t wl, uint64_t v)
//#endif

#ifdef RUNTEST
int varint2mem64_0(uint8_t *wire, ssize_t wl, uint64_t v)
#endif
#if defined RUNTEST || defined V2W0
{
	int n = 0;
	if (wl <= 0)
		return 0;
	do {
		uint8_t u8 = v & 0x7f;
		v >>= 7;
		if (v)
			u8 |= 0x80;
		*wire = u8;
		++wire;
		--wl;
		++n;
	} while (v && wl);
	if (v)
		return 0;
	return n;
}
#endif

#ifdef RUNTEST
int varint2mem64_0e(uint8_t *wire, uint8_t *e, uint64_t v)
#endif
#if defined RUNTEST || defined V2W0
{
	uint8_t *w = wire;
	do {
		if (w >= e)
			return 0;
		uint8_t u8 = v & 0x7f;
		v >>= 7;
		if (v)
			u8 |= 0x80;
		*w++ = u8;
	} while (v);
	return w-wire;
}
#endif

#ifdef RUNTEST
int varint2mem64_1(uint8_t *wire, ssize_t wl, uint64_t v)
#endif
#if defined RUNTEST || defined V2W1
{
	uint8_t *w = wire, *e = wire + wl;
	do {
		if (wire >= e)
			return 0;
		uint8_t u8 = v & 0x7f;
		v >>= 7;
		if (v)
			u8 |= 0x80;
		*wire = u8;
		++wire;
	} while (v);
	return wire-w;
}
#endif


#ifdef RUNTEST
int varint2mem64_2(uint8_t *wire, ssize_t wl, uint64_t v)
#endif
#if defined RUNTEST || defined V2W1
{
	uint8_t *w = wire;
	do {
		if (--wl < 0)
			return 0;
		uint8_t u8 = v & 0x7f;
		v >>= 7;
		if (v)
			u8 |= 0x80;
		*wire = u8;
		++wire;
	} while (v);
	return wire-w;
}
#endif


#ifdef RUNTEST
int varint2mem64_3(uint8_t *wire, ssize_t wl, uint64_t v)
#endif
#if defined RUNTEST || defined V2W0
{
	int n = 0;
	while ((wl > 0) && (v != 0)) {
		uint8_t u8 = v & 0x7f;
		v >>= 7;
		if (v)
			u8 |= 0x80;
		*wire = u8;
		++wire;
		--wl;
		++n;
	}
	assert(v == 0);
	return n;
}
#endif

//#ifndef RUNTEST
//int varint2mem32(uint8_t *wire, ssize_t wl, uint32_t v)
//#endif

#ifdef RUNTEST
int varint2mem32_0(uint8_t *wire, ssize_t wl, uint32_t v)
#endif
#if defined RUNTEST || defined V2W0
{
	int n = 0;
	if (wl <= 0)
		return 0;
	do {
		uint8_t u8 = v & 0x7f;
		v >>= 7;
		if (v)
			u8 |= 0x80;
		*wire = u8;
		++wire;
		--wl;
		++n;
	} while (v && wl);
	if (v)
		return 0;
	return n;
}
#endif

#ifdef RUNTEST
int varint2mem32_1(uint8_t *wire, ssize_t wl, uint32_t v)
#endif
#if defined RUNTEST || defined V2W1
{
	uint8_t *w = wire, *e = wire + wl;
	do {
		if (wire >= e)
			return 0;
		uint8_t u8 = v & 0x7f;
		v >>= 7;
		if (v)
			u8 |= 0x80;
		*wire = u8;
		++wire;
	} while (v);
	return wire-w;
}
#endif


//#ifndef RUNTEST
//int varint2mem16(uint8_t *wire, ssize_t wl, uint16_t v)
//#endif

#ifdef RUNTEST
int varint2mem16_0(uint8_t *wire, ssize_t wl, uint16_t v)
#endif
#if defined RUNTEST || defined V2W0
{
	int n = 0;
	if (wl <= 0)
		return 0;
	do {
		uint8_t u8 = v & 0x7f;
		v >>= 7;
		if (v)
			u8 |= 0x80;
		*wire = u8;
		++wire;
		--wl;
		++n;
	} while (v && wl);
	if (v)
		return 0;
	return n;
}
#endif

#ifdef RUNTEST
int varint2mem16_1(uint8_t *wire, ssize_t wl, uint16_t v)
#endif
#if defined RUNTEST || defined V2W1
{
	uint8_t *w = wire, *e = wire + wl;
	do {
		if (wire >= e)
			return 0;
		uint8_t u8 = v & 0x7f;
		v >>= 7;
		if (v)
			u8 |= 0x80;
		*wire = u8;
		++wire;
	} while (v);
	return wire-w;
}
#endif


/*	dec	bin	varint
 * 	-1	1..111	0..001
 * 	-2	1..110	0..011
 */
// moved to helper
#ifdef RUNTEST
int16_t varint_sint16(uint64_t v)
{
	if (v & 1)
		return ~(v >> 1);
	else
		return v >> 1;
}


int32_t varint_sint32(uint64_t v)
{
	if (v & 1)
		return ~(v >> 1);
	else
		return v >> 1;
}


/* opt for review: speed @ -g */
int64_t varint_sint64_0(uint64_t v)
{
	if (v & 1)
		return ~(v >> 1);
	else
		return v >> 1;
}

/* opt for size+speed @ -O2 -Os */
int64_t varint_sint64_1(uint64_t v)
{
	uint64_t x = -(v & 1);
	v >>= 1;
	v ^= x;
	return v;
}

/* opt for speed @ arm */
int64_t varint_sint64_2(uint64_t v)
{
	uint8_t x = v & 1;
	v >>= 1;
	if (x)
		v = ~v;
	return v;
}


uint64_t sint64_varint(int64_t i)
{
	return (i << 1) ^ (i >> 63);
}
#endif


//#ifndef RUNTEST
//unsigned wiresize_u64(uint64_t u)
//#endif
#ifdef WFC
unsigned wiresize_u64(uint64_t u)
#endif

#if defined RUNTEST
unsigned wiresize_u64_0(uint64_t u)
#endif
#if defined RUNTEST || defined WSU64_0
{
	if (u < (1ULL << 7))
		return 1;
	if (u < (1ULL << 14))
		return 2;
	if (u < (1ULL << 21))
		return 3;
	if (u < (1ULL << 28))
		return 4;
	if (u < (1ULL << 35))
		return 5;
	if (u < (1ULL << 42))
		return 6;
	if (u < (1ULL << 49))
		return 7;
	if (u < (1ULL << 56))
		return 8;
	if (u < (1ULL << 63))
		return 9;
	return 10;
}
#endif


#if defined RUNTEST
unsigned wiresize_u64_1(uint64_t u)
#endif
#if defined RUNTEST || defined WSU64_5
{
	if (0 == (u>>=7U))
		return 1;
	if (0 == (u>>=7U))
		return 2;
	if (0 == (u>>=7U))
		return 3;
	if (0 == (u>>=7U))
		return 4;
	if (0 == (u>>=7U))
		return 5;
	if (0 == (u>>=7U))
		return 6;
	if (0 == (u>>=7U))
		return 7;
	if (0 == (u>>=7U))
		return 8;
	if (0 == (u>>=7U))
		return 9;
	return 10;
}
#endif


#if defined RUNTEST
unsigned wiresize_u64_2(uint64_t u)
#endif
#if defined RUNTEST || defined WSU64_1
{
	if ((u >> 7) == 0)
		return 1;
	if ((u >> 14) == 0)
		return 2;
	if ((u >> 21) == 0)
		return 3;
	if ((u >> 28) == 0)
		return 4;
	if ((u >> 35) == 0)
		return 5;
	if ((u >> 42) == 0)
		return 6;
	if ((u >> 49) == 0)
		return 7;
	if ((u >> 56) == 0)
		return 8;
	if ((u >> 63) == 0)
		return 9;
	return 10;
}
#endif


#if defined RUNTEST
unsigned wiresize_u64_3(uint64_t u)
#endif
#if defined RUNTEST || defined WSU64_3
{
	unsigned x = 0;
	do ++x;
	while (u>>=7U);
	return x;
}
#endif


#if defined RUNTEST
unsigned wiresize_u64_4(uint64_t u)
#endif
#if defined RUNTEST || defined WSU64_4
{
	unsigned x = 1;
	while (u>>=7U)
		++x;
	return x;
}
#endif


//#ifndef RUNTEST
//unsigned wiresize_s64(int64_t s)
//{
//	uint64_t u = (s << 1) ^ (s >> 63);
//	return wiresize_u64(u);
//}
//#endif



//#ifndef RUNTEST
//unsigned wiresize_u32(uint32_t u)
//#endif

#if defined RUNTEST
unsigned wiresize_u32_0(uint32_t u)
#endif
#if defined RUNTEST || defined WSU32_0
{
	if (u < (1ULL << 7))
		return 1;
	if (u < (1ULL << 14))
		return 2;
	if (u < (1ULL << 21))
		return 3;
	if (u < (1ULL << 28))
		return 4;
	return 5;
}
#endif


#if defined RUNTEST
unsigned wiresize_u32_1(uint32_t u)
#endif
#if defined RUNTEST || defined WSU32_1
{
	unsigned x = 0;
	do ++x;
	while (u>>=7U);
	return x;
}
#endif

//#ifndef RUNTEST
//unsigned wiresize_s32(int32_t s)
//{
//	uint32_t u = (s << 1) ^ (s >> 31);
//	return wiresize_u32(u);
//}
//#endif


//#ifndef RUNTEST
//unsigned wiresize_u16(uint16_t u)
//#endif

#if defined RUNTEST
unsigned wiresize_u16_0(uint16_t u)
#endif
#if defined RUNTEST || defined WSU16_0
{
	if (u < (1ULL << 7))
		return 1;
	if (u < (1ULL << 14))
		return 2;
	return 3;
}
#endif




#if defined RUNTEST
unsigned wiresize_u16_1(uint16_t u)
#endif
#if defined RUNTEST || defined WSU16_1
{
	unsigned x = 0;
	do ++x;
	while (u>>=7U);
	return x;
}
#endif


//#ifndef RUNTEST
//unsigned wiresize_s16(int16_t s)
//{
//	uint16_t u = (s << 1) ^ (s >> 15);
//	return wiresize_u16(u);
//}
//#endif

#ifdef RUNTEST
void get_u16(const uint8_t *wire, uint16_t *r)
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	*r = *(uint16_t *)wire;
#elif __BYTE_ORDER == __BIG_ENDIAN
	*r = (uint16_t)wire[0];
	*r |= (uint16_t)wire[1] << 8;
#else
#error unknown endian
#endif
}
#endif


#if defined(RUNTEST) || defined(WFC)
unsigned wire2u16(const uint8_t *wire, signed wl, uint16_t *r)
{
	if (wl < 2)
		return 0;
#if __BYTE_ORDER == __LITTLE_ENDIAN
	*r = *(uint16_t *)wire;
#elif __BYTE_ORDER == __BIG_ENDIAN
	*r = (uint16_t)wire[0];
	*r |= (uint16_t)wire[1] << 8;
#else
#error unknown endian
#endif
	return 2;
}
#endif


#ifdef RUNTEST
void get_u32(const uint8_t *wire, uint32_t *r)
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	*r = *(uint32_t *)wire;
#elif __BYTE_ORDER == __BIG_ENDIAN
	*r = (uint32_t)wire[0];
	*r |= (uint32_t)wire[1] << 8;
	*r |= (uint32_t)wire[2] << 16;
	*r |= (uint32_t)wire[3] << 24;
#else
#error unknown endian
#endif
}
#endif


#if defined(RUNTEST) || defined(WFC)
unsigned wire2u32(const uint8_t *wire, signed wl, uint32_t *r)
{
	if (wl < 4)
		return 0;
#if __BYTE_ORDER == __LITTLE_ENDIAN
	*r = *(uint32_t *)wire;
#elif __BYTE_ORDER == __BIG_ENDIAN
	*r = (uint32_t)wire[0];
	*r |= (uint32_t)wire[1] << 8;
	*r |= (uint32_t)wire[2] << 16;
	*r |= (uint32_t)wire[3] << 24;
#else
#error unknown endian
#endif
	return 4;
}
#endif


#if __BYTE_ORDER == __BIG_ENDIAN || defined(RUNTEST)
uint16_t read_u16_0(const uint8_t *wire)
{
	uint16_t r;
	r = (uint16_t)*wire++;
	r |= (uint16_t)*wire++ << 8;
	return r;
}

uint16_t read_u16_1(const uint8_t *wire)
{
	uint16_t r;
	r	= (uint16_t)wire[0]
		| (uint16_t)wire[1] << 8;
	return r;
}

uint32_t read_u32_0(const uint8_t *wire)
{
	uint32_t r;
	r = (uint32_t)*wire++;
	r |= (uint32_t)*wire++ << 8;
	r |= (uint32_t)*wire++ << 16;
	r |= (uint32_t)*wire << 24;
	return r;
}

float read_float_0(const uint8_t *wire)
{
	union { uint32_t i; float f; } r;
	r.i = (uint32_t)*wire++;
	r.i |= (uint32_t)*wire++ << 8;
	r.i |= (uint32_t)*wire++ << 16;
	r.i |= (uint32_t)*wire << 24;
	return r.f;
}

uint64_t read_u64_0(const uint8_t *wire)
{
	uint64_t r;
	r	= (uint64_t)wire[0]
		| (uint64_t)wire[1] << 8
		| (uint64_t)wire[2] << 16
		| (uint64_t)wire[3] << 24
		| (uint64_t)wire[4] << 32
		| (uint64_t)wire[5] << 40
		| (uint64_t)wire[6] << 48
		| (uint64_t)wire[7] << 56;
	return r;
}

uint64_t read_u64_1(const uint8_t *wire)
{
	uint64_t r;
	r = (uint64_t)*wire++;
	r |= (uint64_t)*wire++ << 8;
	r |= (uint64_t)*wire++ << 16;
	r |= (uint64_t)*wire++ << 24;
	r |= (uint64_t)*wire++ << 32;
	r |= (uint64_t)*wire++ << 40;
	r |= (uint64_t)*wire++ << 48;
	r |= (uint64_t)*wire << 56;
	return r;
}

uint64_t read_u64_2(const uint8_t *wire)
{
	uint64_t r = 0;
	for (int i = 0 ; i < 64; i+=8)
		r |= (uint64_t)*wire++ << i;
	return r;
}


double read_double_0(const uint8_t *wire)
{
	union { uint64_t i; double f; } r;
	r.i = (uint64_t)*wire++;
	r.i |= (uint64_t)*wire++ << 8;
	r.i |= (uint64_t)*wire++ << 16;
	r.i |= (uint64_t)*wire++ << 24;
	r.i |= (uint64_t)*wire++ << 32;
	r.i |= (uint64_t)*wire++ << 40;
	r.i |= (uint64_t)*wire++ << 48;
	r.i |= (uint64_t)*wire << 56;
	return r.f;
}

#endif


#ifdef RUNTEST
void get_u64(const uint8_t *wire, uint64_t *r)
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	*r = *(uint64_t *)wire;
#elif __BYTE_ORDER == __BIG_ENDIAN
	*r = (uint64_t)wire[0];
	*r |= (uint64_t)wire[1] << 8;
	*r |= (uint64_t)wire[2] << 16;
	*r |= (uint64_t)wire[3] << 24;
	*r |= (uint64_t)wire[4] << 32;
	*r |= (uint64_t)wire[5] << 40;
	*r |= (uint64_t)wire[6] << 48;
	*r |= (uint64_t)wire[7] << 56;
#else
#error unknown endian
#endif
}
#endif


#if defined(RUNTEST) || defined(WFC)
unsigned wire2u64(const uint8_t *wire, signed wl, uint64_t *r)
{
	if (wl < 8)
		return 0;
#if __BYTE_ORDER == __LITTLE_ENDIAN
	*r = *(uint64_t *)wire;
#elif __BYTE_ORDER == __BIG_ENDIAN
	*r = (uint64_t)wire[0];
	*r |= (uint64_t)wire[1] << 8;
	*r |= (uint64_t)wire[2] << 16;
	*r |= (uint64_t)wire[3] << 24;
	*r |= (uint64_t)wire[4] << 32;
	*r |= (uint64_t)wire[5] << 40;
	*r |= (uint64_t)wire[6] << 48;
	*r |= (uint64_t)wire[7] << 56;
#else
#error unknown endian
#endif
	return 8;
}
#endif


/*
unsigned u16_mem(uint8_t *wire, ssize_t wl, uint16_t v)
{
	if (wl < 2)
		return 0;
#if __BYTE_ORDER == __LITTLE_ENDIAN
	*(uint16_t*)wire = v;
#elif __BYTE_ORDER == __BIG_ENDIAN
	*wire++ = v & 0xff;
	v >>= 8;
	*wire++ = v & 0xff;
#else
#error unknown endian
#endif
	return 2;
}


unsigned u32_mem(uint8_t *wire, ssize_t wl, uint32_t v)
{
	if (wl < 4)
		return 0;
#if __BYTE_ORDER == __LITTLE_ENDIAN
	*(uint32_t*)wire = v;
#elif __BYTE_ORDER == __BIG_ENDIAN
	*wire++ = v & 0xff;
	v >>= 8;
	*wire++ = v & 0xff;
	v >>= 8;
	*wire++ = v & 0xff;
	v >>= 8;
	*wire++ = v & 0xff;
#else
#error unknown endian
#endif
	return 4;
}


unsigned u64_mem(uint8_t *wire, ssize_t wl, uint64_t v)
{
	if (wl < 8)
		return 0;
#if __BYTE_ORDER == __LITTLE_ENDIAN
	*(uint64_t*)wire = v;
#elif __BYTE_ORDER == __BIG_ENDIAN
	*wire++ = v & 0xff;
	v >>= 8;
	*wire++ = v & 0xff;
	v >>= 8;
	*wire++ = v & 0xff;
	v >>= 8;
	*wire++ = v & 0xff;
	v >>= 8;
	*wire++ = v & 0xff;
	v >>= 8;
	*wire++ = v & 0xff;
	v >>= 8;
	*wire++ = v & 0xff;
	v >>= 8;
	*wire++ = v & 0xff;
#else
#error unknown endian
#endif
	return 8;
}
*/


/*
unsigned u16_wire(void (*put)(uint8_t), uint16_t v)
{
	put(v & 0xff);
	v >>= 8;
	put(v);
	return 2;
}


unsigned u32_wire(void (*put)(uint8_t), uint32_t v)
{
	put(v & 0xff);
	v >>= 8;
	put(v & 0xff);
	v >>= 8;
	put(v & 0xff);
	v >>= 8;
	put(v);
	return 4;
}


unsigned u64_wire(void (*put)(uint8_t), uint64_t v)
{
	put(v & 0xff);
	v >>= 8;
	put(v & 0xff);
	v >>= 8;
	put(v & 0xff);
	v >>= 8;
	put(v & 0xff);
	v >>= 8;
	put(v & 0xff);
	v >>= 8;
	put(v & 0xff);
	v >>= 8;
	put(v & 0xff);
	v >>= 8;
	put(v);
	return 8;
}
*/


unsigned skip_field(const uint8_t *wire, signed wl)
{
	if (wl <= 0)
		return 0;
	unsigned n = 1;
	switch (*wire++&0x7) {
	case 0:
		while (--wl && (*wire++&0x80)) ++n;
		return n;
	case 1:
		if (wl < 9)
			return wl;
		return 9;
	case 2: {
			varint_t l = 0;
			n = wire2varint(wire,wl,&l);
			if (n == 0)
				return wl;
			l += n + 1;
			if ((signed)l < wl)
				return l;
			return wl;
		}
	case 5:
		if (wl < 5)
			return wl;
		return 5;
	default:
		assert(0);
	}
	return 0;
}


#ifdef RUNTEST
unsigned skip_field_xproto(const uint8_t *wire, size_t wl)
{
	if (wl <= 0)
		return 0;
	unsigned n = 1;
	switch (*wire++&0x7) {
	case 0:
		while (--wl && (*wire++&0x80)) ++n;
		return n;
	case 1:
		if (wl < 9)
			return wl;
		return 9;
	case 2: {
			varint_t l = 0;
			n = wire2varint(wire,wl,&l);
			if (n == 0)
				return wl;
			l += n + 1;
			if (l < wl)
				return l;
			return wl;
		}
	case 3:
		if (wl < 2)
			return wl;
		return 2;
	case 4:
		if (wl < 3)
			return wl;
		return 3;
	case 5:
		if (wl < 5)
			return wl;
		return 5;
	default:
		assert(0);
	}
}
#endif

#ifdef RUNTEST
#include <assert.h>
#include <stdio.h>
#if defined __x86 || defined __x86_64
inline uint64_t rdtsc()
{
	uint64_t msr;
	asm volatile (
		"cpuid\n\t"
		"rdtsc\n\t"    // Returns the time in EDX:EAX.
		"shl $32, %%rdx\n\t"  // Shift the upper bits left.
		"or %%rdx, %0"        // 'Or' in the lower bits.
		: "=a" (msr)
		:
		: "rdx");
	return msr;
}
/*
	uint32_t lower = 0, upper = 0;
	asm volatile ("cpuid\nrdtsc": "=a" (lower), "=d" (upper));
	uint64_t ull = ((uint64_t) upper << 32) + lower;
	return ull;
*/
#elif defined _POSIX_TIMERS && (_POSIX_TIMERS > 0)
#include <time.h>
inline uint64_t rdtsc()
{
	struct timespec t;
	clock_gettime(CLOCK_REALTIME,&t);
	uint64_t r = t.tv_nsec/1000000 + t.tv_sec*1000;
	return r;
}
#else
#define rdtsc() 0
#endif


uint64_t perftest_wsu64_0()
{
	uint64_t t = rdtsc();
	uint64_t a = 0;
	uint64_t v = 0;
	while (a < 0x70000000ULL)
		v ^= wiresize_u64_0(++a);
	printf("wiresize_u64_0=%lx: %lu\n",v,rdtsc()-t);
	return v;
}

uint64_t perftest_wsu64_1()
{
	uint64_t t = rdtsc();
	uint64_t a = 0;
	uint64_t v = 0;
	while (a < 0x70000000ULL)
		v ^= wiresize_u64_1(++a);
	printf("wiresize_u64_1=%lx: %lu\n",v,rdtsc()-t);
	return v;
}

uint64_t perftest_wsu64_2()
{
	uint64_t t = rdtsc();
	uint64_t a = 0;
	uint64_t v = 0;
	while (a < 0x70000000ULL)
		v ^= wiresize_u64_2(++a);
	printf("wiresize_u64_2=%lx: %lu\n",v,rdtsc()-t);
	return v;
}

uint64_t perftest_wsu64_3()
{
	uint64_t t = rdtsc();
	uint64_t a = 0;
	uint64_t v = 0;
	while (a < 0x70000000ULL)
		v ^= wiresize_u64_3(++a);
	printf("wiresize_u64_3=%lx: %lu\n",v,rdtsc()-t);
	return v;
}

uint64_t perftest_wsu64_4()
{
	uint64_t t = rdtsc();
	uint64_t a = 0;
	uint64_t v = 0;
	while (a < 0x70000000ULL)
		v ^= wiresize_u64_4(++a);
	printf("wiresize_u64_4=%lx: %lu\n",v,rdtsc()-t);
	return v;
}


/*
void perftest_w2vi64_0()
{
	uint8_t buf[16];
	t = rdtsc();
	for (volatile int i = 0; i < 0x7000000ULL; ++i)
		wire2varint_0(buf,sizeof(buf),&a);
	printf("wire2varint_0: %lu\n",rdtsc()-t);
}
*/

#if VARINTBITS==64
#define varint2mem_0 varint2mem64_0
#define varint2mem_1 varint2mem64_1
#define wire2varint_0 wire2varint64_0
#define wire2varint_1 wire2varint64_1
#define wire2varint_2 wire2varint64_2
#define wire2varint_3 wire2varint64_3
#endif

uint64_t perftest_v2w_w2v_0_0()
{
	uint64_t t = rdtsc();
	uint64_t result = 0;
	for (uint64_t i = 0; i < 0x1000000000ULL; i += 117) {
		uint8_t buf[16];
		varint_t a;
		result ^= i;
		varint2mem_0(buf,sizeof(buf),i);
		wire2varint_0(buf,sizeof(buf),&a);
		result ^= a;
	}
	printf("perftest_v2w_w2v_0_0 %lu (0 expected): %ld\n",result,rdtsc()-t);
	return result;
}

uint64_t perftest_v2w_w2v_0_1()
{
	uint64_t t = rdtsc();
	uint64_t result = 0;
	for (uint64_t i = 0; i < 0x1000000000ULL; i += 117) {
		uint8_t buf[16];
		varint_t a;
		result ^= i;
		varint2mem_0(buf,sizeof(buf),i);
		wire2varint_1(buf,sizeof(buf),&a);
		result ^= a;
	}
	printf("perftest_v2w_w2v_0_1 %lu (0 expected): %ld\n",result,rdtsc()-t);
	return result;
}


uint64_t perftest_v2w_w2v_0_2()
{
	uint64_t t = rdtsc();
	varint_t result = 0;
	for (uint64_t i = 0; i < 0x1000000000ULL; i += 117) {
		uint8_t buf[16];
		varint_t a;
		result ^= i;
		varint2mem_0(buf,sizeof(buf),i);
		wire2varint_2(buf,sizeof(buf),&a);
		result ^= a;
	}
	printf("perftest_v2w_w2v_0_2 %lu (0 expected): %ld\n",result,rdtsc()-t);
	return result;
}


uint64_t perftest_v2w_w2v_0_3()
{
	uint64_t t = rdtsc();
	varint_t result = 0;
	for (uint64_t i = 0; i < 0x1000000000ULL; i += 117) {
		uint8_t buf[16];
		varint_t a;
		result ^= i;
		varint2mem_0(buf,sizeof(buf),i);
		wire2varint_3(buf,sizeof(buf),&a);
		result ^= a;
	}
	printf("perftest_v2w_w2v_0_3 %lu (0 expected): %ld\n",result,rdtsc()-t);
	return result;
}


uint64_t perftest_v2w_w2v_1_0()
{
	uint64_t t = rdtsc();
	varint_t result = 0;
	for (uint64_t i = 0; i < 0x1000000000ULL; i += 117) {
		uint8_t buf[16];
		varint_t a;
		result ^= i;
		varint2mem_1(buf,sizeof(buf),i);
		wire2varint_0(buf,sizeof(buf),&a);
		result ^= a;
	}
	printf("perftest_v2w_w2v_1_0 %lu (0 expected): %ld\n",result,rdtsc()-t);
	return result;
}


uint64_t perftest_v2w_w2v_1_1()
{
	uint64_t t = rdtsc();
	varint_t result = 0;
	for (uint64_t i = 0; i < 0x1000000000ULL; i += 117) {
		uint8_t buf[16];
		varint_t a;
		result ^= i;
		varint2mem_1(buf,sizeof(buf),i);
		wire2varint_1(buf,sizeof(buf),&a);
		result ^= a;
	}
	printf("perftest_v2w_w2v_1_1 %lu (0 expected): %ld\n",result,rdtsc()-t);
	return result;
}


uint64_t perftest_v2w_w2v_1_2()
{
	uint64_t t = rdtsc();
	varint_t result = 0;
	for (uint64_t i = 0; i < 0x1000000000ULL; i += 117) {
		uint8_t buf[16];
		varint_t a;
		result ^= i;
		varint2mem_1(buf,sizeof(buf),i);
		wire2varint_2(buf,sizeof(buf),&a);
		result ^= a;
	}
	printf("perftest_v2w_w2v_1_2 %lu (0 expected): %ld\n",result,rdtsc()-t);
	return result;
}


uint64_t perftest_v2w_w2v_1_3()
{
	uint64_t t = rdtsc();
	varint_t result = 0;
	for (uint64_t i = 0; i < 0x1000000000ULL; i += 117) {
		uint8_t buf[16];
		varint_t a;
		result ^= i;
		varint2mem_1(buf,sizeof(buf),i);
		wire2varint_3(buf,sizeof(buf),&a);
		result ^= a;
	}
	printf("perftest_v2w_w2v_1_3 %lu (0 expected): %ld\n",result,rdtsc()-t);
	return result;
}


//volatile bool Stop = false;
uint64_t perftest_v2w_w2v(void (*v2w)(char*,size_t,uint64_t),void(*w2v)(char*,size_t,varint_t *),int a, int b)
{
	uint64_t t = rdtsc();
	varint_t result = 0;
	for (uint64_t i = 0; i < 0x1000000000ULL; i += 117) {
		char buf[16];
		varint_t a;
		result ^= i;
		v2w(buf,sizeof(buf),i);
		w2v(buf,sizeof(buf),&a);
		result ^= a;
	}
	printf("%s_%d_%d %lu (0 expected): %ld\n",__FUNCTION__,a,b,result,rdtsc()-t);
	return result;
}


void varint_sint_test()
{
	int64_t values[] = {0,-1,1,10,-10,1000,-1000,1024,-1024,INT64_MIN,INT64_MAX};

	for (int i = 0; i < sizeof(values)/sizeof(values[0]); ++i) {
		int64_t vi = values[i];
		varint_t w = sint64_varint(vi);
		int64_t vo0 = varint_sint64_0(w);
		assert(vi == vo0);
		int64_t vo1 = varint_sint64_1(w);
		assert(vi == vo1);
		int64_t vo2 = varint_sint64_2(w);
		assert(vi == vo2);
	}
	printf("varint_sint_test(): OK\n");
}


void perftest_varint_sint(int64_t (*varint_sint)(uint64_t))
{
	static unsigned x = 0;
	uint64_t t = rdtsc();
	volatile varint_t o;
	for (int64_t i = -10000000000; i < 10000000000; i += 73) {
		varint_t w = sint64_varint(i);
		o = varint_sint(w);
	}
	printf("%s_%d: %ld\n",__FUNCTION__,x++,rdtsc()-t);
}



int main()
{
	varint_sint_test();
	uint64_t x = 1;
	uint32_t b = 0;
	uint8_t buf[16];
	while (x) {
		unsigned r[8];
		r[0] = wiresize_u64_0(x);
		r[1] = wiresize_u64_1(x);
		r[2] = wiresize_u64_2(x);
		r[3] = wiresize_u64_3(x);
		r[4] = wiresize_u64_4(x);
		if ((r[0] == r[1]) && (r[2] == r[3]) && (r[4] == r[5]) && (r[1] == r[2]) && (r[3] == r[4])) {
			printf("wiresize_u64: OK\n");
		} else {
			printf("wiresize_u64: INCONSISTENCY\n");
			printf("wiresize_u64_0(%lx) = %d\n",x,r[0]);
			printf("wiresize_u64_1(%lx) = %d\n",x,r[1]);
			printf("wiresize_u64_2(%lx) = %d\n",x,r[2]);
			printf("wiresize_u64_3(%lx) = %d\n",x,r[3]);
			printf("wiresize_u64_4(%lx) = %d\n",x,r[4]);
		}
		r[0] = wiresize_u32_0(x);
		r[1] = wiresize_u32_1(x);
		if (r[0] == r[1]) {
			printf("wiresize_u32: OK\n");
		} else {
			printf("wiresize_u32_0(%lx) = %d\n",x,r[0]);
			printf("wiresize_u32_1(%lx) = %d\n",x,r[1]);
		}
		uint8_t buf1[16];
		int n0 = varint2mem_0(buf,sizeof(buf),x);
		int n1 = varint2mem_1(buf1,sizeof(buf1),x);
		if ((n0 != n1) && (0 == memcmp(buf,buf1,n0))) {
			printf("varint2mem():");
			int i;
			for (i = 0; i < sizeof(buf); ++i) {
				printf(" %02hhx",buf[i]);
				if ((buf[i] & 0x80) == 0)
					break;
			}
			printf("\n");
		} else {
			printf("varint2mem(): OK\n");
		}
		int n;
		varint_t v0;
		n = wire2varint_0(buf,sizeof(buf),&v0);
		assert(v0 == x);
		varint_t v1;
		n = wire2varint_1(buf,sizeof(buf),&v1);
		assert(v1 == x);
		varint_t v2;
		n = wire2varint_2(buf,sizeof(buf),&v2);
		assert(v2 == x);
		varint_t v3;
		n = wire2varint_3(buf,sizeof(buf),&v3);
		if ((v0 != v1) || (v0 != v2) || (v0 != v3)) {
			printf("wire2varint: INCONSISTENCY\n");
			printf("wire2varint_0: %lx (%d)\n",v0,n);
			printf("wire2varint_1: %lx (%d)\n",v1,n);
			printf("wire2varint_2: %lx (%d)\n",v2,n);
			printf("wire2varint_3: %lx (%d)\n",v3,n);
		} else {
			printf("wire2varint: OK\n");
		}
		assert(v3 == x);

		x <<= 1;
	}
	perftest_varint_sint(varint_sint64_0);
	perftest_varint_sint(varint_sint64_1);
	perftest_varint_sint(varint_sint64_2);
	perftest_v2w_w2v_0_0();
	perftest_v2w_w2v_0_1();
	perftest_v2w_w2v_0_2();
	perftest_v2w_w2v_0_3();
	perftest_v2w_w2v_1_0();
	perftest_v2w_w2v_1_1();
	perftest_v2w_w2v_1_2();
	perftest_v2w_w2v_1_3();
	perftest_wsu64_0();
	perftest_wsu64_1();
	perftest_wsu64_2();
	perftest_wsu64_3();
	perftest_wsu64_4();
}
#endif
