/*
 *  Copyright (C) 2017-2018, Thomas Maier-Komor
 *
 *  This source file belongs to Wire-Format-Compiler.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef WIREFUNCS_H
#define WIREFUNCS_H

#include <stdint.h>

#ifdef __AVR__	// AVR
typedef long unsigned uint32_t;
#include <inttypes.h>
#define __LITTLE_ENDIAN 1
#define __BYTE_ORDER __LITTLE_ENDIAN
#ifndef VARINTBITS
#define VARINTBITS 16
typedef uint16_t varint_t;
#endif

#elif defined __XTENSA__	// ESP8266
#include <inttypes.h>
#ifdef __XTENSA_EL__
#define __LITTLE_ENDIAN 1
#define __BYTE_ORDER __LITTLE_ENDIAN
#endif
#ifndef VARINTBITS
#define VARINTBITS 32
typedef uint32_t varint_t;
#endif

#else // unknown architecture
#include <sys/types.h>
#ifndef VARINTBITS
#define VARINTBITS 64
typedef uint64_t varint_t;
#endif
#endif

#if !defined(__BYTE_ORDER) && defined(__BYTE_ORDER__)
#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
#ifndef __LITTLE_ENDIAN
#define __LITTLE_ENDIAN __ORDER_LITTLE_ENDIAN__
#endif
#define __BYTE_ORDER __LITTLE_ENDIAN
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
#define __BIG_ENDIAN __ORDER_BIG_ENDIAN__
#define __BYTE_ORDER __BIG_ENDIAN
#else
#error unknown byte order
#endif
#endif

#ifndef __BYTE_ORDER
#error byte order not defined
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifdef VARINTBITS
#if VARINTBITS==64
int wire2varint(const uint8_t *wire, signed wl, uint64_t *r);
int varint2mem(uint8_t *wire, signed wl, uint64_t v);
#elif VARINTBITS==32
int wire2varint(const uint8_t *wire, signed wl, uint32_t *r);
int varint2mem(uint8_t *wire, signed wl, uint32_t v);
#define mem2varint32 wire2varint 
#elif VARINTBITS==16
#define mem2varint16 wire2varint 
int wire2varint(const uint8_t *wire, signed wl, uint16_t *r);
int varint2mem(uint8_t *wire, signed wl, uint16_t v);
#endif
#endif // VARINTBITS

unsigned wire2u16(const uint8_t *wire, signed wl, uint16_t *r);
unsigned wire2u32(const uint8_t *wire, signed wl, uint32_t *r);
unsigned wire2u64(const uint8_t *wire, signed wl, uint64_t *r);

int wire2int32(const uint8_t *wire, signed wl, int32_t *r);
unsigned skip_field(const uint8_t *wire, signed wl);
unsigned wiresize_u64(uint64_t u);

#if __BYTE_ORDER == __LITTLE_ENDIAN
#define read_u16(wire) (*(uint16_t *)(wire))
#define read_u32(wire) (*(uint32_t *)(wire))
#define read_u64(wire) (*(uint64_t *)(wire))
#define read_float(wire) (*(float *)(wire))
#define read_double(wire) (*(double *)(wire))
#elif __BYTE_ORDER == __BIG_ENDIAN
uint16_t read_u16(const uint8_t *);
uint32_t read_u32(const uint8_t *);
uint64_t read_u64(const uint8_t *);
float read_float(const uint8_t *);
double read_double(const uint8_t *);
#else
#error unknown endian
#endif

#ifdef __cplusplus
}	// extern "C"
#endif

#endif
